@@ -15,6 +15,19 @@
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of portions of this program with the
+  OpenSSL library under certain conditions as described in each
+  individual source file, and distribute linked combinations
+  including the two.
+  You must obey the GNU General Public License in all respects
+  for all of the code used other than OpenSSL.  If you modify
+  file(s) with this exception, you may extend this exception to your
+  version of the file(s), but you are not obligated to do so.  If you
+  do not wish to do so, delete this exception statement from your
+  version.  If you delete this exception statement from all source
+  files in the program, then also delete it here.
 */
 
 
@@ -397,7 +410,7 @@ extern "C" {
 /**
  * Wait for next eXosip event.
  * **DEPRECATED API**
- * This API will block // You should use eXosip_event_wait instead which is
+ * This API will block - You should use eXosip_event_wait instead which is
  * more convenient.
  * limitation: This method will not process automatic 200ok retransmission
  * for INVITE transaction.
