@@ -15,6 +15,19 @@
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of portions of this program with the
+  OpenSSL library under certain conditions as described in each
+  individual source file, and distribute linked combinations
+  including the two.
+  You must obey the GNU General Public License in all respects
+  for all of the code used other than OpenSSL.  If you modify
+  file(s) with this exception, you may extend this exception to your
+  version of the file(s), but you are not obligated to do so.  If you
+  do not wish to do so, delete this exception statement from your
+  version.  If you delete this exception statement from all source
+  files in the program, then also delete it here.
 */
 
 #include "eXosip2.h"
@@ -233,7 +246,6 @@ eXosip_call_send_initial_invite (struct eXosip_t *excontext, osip_message_t * in
   int i;
 
   if (invite == NULL) {
-    osip_message_free (invite);
     return OSIP_BADPARAMETER;
   }
 
@@ -358,8 +370,11 @@ eXosip_call_send_ack (struct eXosip_t *excontext, int did, osip_message_t * ack)
   char *host = NULL;
   int port;
 
-  if (did <= 0)
+  if (did <= 0) {
+    if (ack != NULL)
+      osip_message_free (ack);
     return OSIP_BADPARAMETER;
+  }
   if (did > 0) {
     _eXosip_call_dialog_find (excontext, did, &jc, &jd);
   }
@@ -762,16 +777,6 @@ eXosip_call_send_answer (struct eXosip_t *excontext, int tid, int status, osip_m
     return OSIP_BADPARAMETER;
   }
 
-  if (0 == osip_strcasecmp (tr->orig_request->sip_method, "INVITE")) {
-    if (MSG_IS_STATUS_2XX (answer) && jd != NULL) {
-      if (status >= 200 && status < 300 && jd != NULL) {
-        _eXosip_dialog_set_200ok (jd, answer);
-        /* wait for a ACK */
-        osip_dialog_set_state (jd->d_dialog, DIALOG_CONFIRMED);
-      }
-    }
-  }
-
   if (0 == osip_strcasecmp (tr->orig_request->sip_method, "INVITE")
       || 0 == osip_strcasecmp (tr->orig_request->sip_method, "UPDATE")) {
     if (MSG_IS_STATUS_2XX (answer) && jd != NULL) {
@@ -784,7 +789,7 @@ eXosip_call_send_answer (struct eXosip_t *excontext, int tid, int status, osip_m
       while (i >= 0) {
         if (supported == NULL)
           break;
-        if (supported->hvalue != NULL && osip_strcasecmp (supported->hvalue, "timer") == 0) {
+        if (supported->hvalue != NULL && strstr (supported->hvalue, "timer") != NULL) {
           /*found */
           break;
         }
@@ -852,7 +857,7 @@ eXosip_call_send_answer (struct eXosip_t *excontext, int tid, int status, osip_m
             while (i >= 0) {
               if (supported == NULL)
                 break;
-              if (supported->hvalue != NULL && osip_strcasecmp (supported->hvalue, "timer") == 0) {
+              if (supported->hvalue != NULL && strstr (supported->hvalue, "timer") != NULL) {
                 /*found */
                 break;
               }
@@ -868,6 +873,16 @@ eXosip_call_send_answer (struct eXosip_t *excontext, int tid, int status, osip_m
     }
   }
 
+  if (0 == osip_strcasecmp (tr->orig_request->sip_method, "INVITE")) {
+    if (MSG_IS_STATUS_2XX (answer) && jd != NULL) {
+      if (status >= 200 && status < 300 && jd != NULL) {
+        _eXosip_dialog_set_200ok (jd, answer);
+        /* wait for a ACK */
+        osip_dialog_set_state (jd->d_dialog, DIALOG_CONFIRMED);
+      }
+    }
+  }
+
   evt_answer = osip_new_outgoing_sipmessage (answer);
   evt_answer->transactionid = tr->transactionid;
 
@@ -1023,7 +1038,8 @@ eXosip_call_build_prack (struct eXosip_t *excontext, int tid, osip_message_t **
   pos = 0;
   while (!osip_list_eol (jd->d_out_trs, pos)) {
     old_prack_tr = (osip_transaction_t *) osip_list_get (jd->d_out_trs, pos);
-    if (old_prack_tr != NULL && 0 == osip_strcasecmp (old_prack_tr->orig_request->sip_method, "PRACK")) {
+
+    if (old_prack_tr != NULL && old_prack_tr->orig_request != NULL && 0 == osip_strcasecmp (old_prack_tr->orig_request->sip_method, "PRACK")) {
       osip_header_t *rack_header = NULL;
 
       osip_message_header_get_byname (old_prack_tr->orig_request, "RAck", 0, &rack_header);
