@@ -15,6 +15,19 @@
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of portions of this program with the
+  OpenSSL library under certain conditions as described in each
+  individual source file, and distribute linked combinations
+  including the two.
+  You must obey the GNU General Public License in all respects
+  for all of the code used other than OpenSSL.  If you modify
+  file(s) with this exception, you may extend this exception to your
+  version of the file(s), but you are not obligated to do so.  If you
+  do not wish to do so, delete this exception statement from your
+  version.  If you delete this exception statement from all source
+  files in the program, then also delete it here.
 */
 
 
@@ -136,7 +149,7 @@
 
 #include "jpipe.h"
 
-#define EXOSIP_VERSION	"4.0.0"
+#define EXOSIP_VERSION	"4.1.0"
 
 #ifdef __cplusplus
 extern "C" {
@@ -176,11 +189,7 @@ extern "C" {
 #endif
 
   void _eXosip_update (struct eXosip_t *excontext);
-#ifndef OSIP_MONOTHREAD
   void _eXosip_wakeup (struct eXosip_t *excontext);
-#else
-#define _eXosip_wakeup(A)  ;
-#endif
 
 #ifndef DEFINE_SOCKADDR_STORAGE
 #define __eXosip_sockaddr sockaddr_storage
@@ -202,6 +211,7 @@ extern "C" {
     time_t d_session_timer_start;       /* session-timer helper */
     int d_session_timer_length;
     int d_refresher;
+    int d_session_timer_use_update;
 
     time_t d_timer;
     int d_count;
@@ -210,7 +220,7 @@ extern "C" {
 
     osip_list_t *d_inc_trs;
     osip_list_t *d_out_trs;
-    int d_retry;                /* avoid too many unsuccessfull retry */
+    int d_retry;                /* avoid too many unsuccessful retry */
     int d_mincseq;              /* remember cseq after PRACK and UPDATE during setup */
 
     eXosip_dialog_t *next;
@@ -225,7 +235,7 @@ extern "C" {
     eXosip_dialog_t *c_dialogs;
     osip_transaction_t *c_inc_tr;
     osip_transaction_t *c_out_tr;
-    int c_retry;                /* avoid too many unsuccessfull retry */
+    int c_retry;                /* avoid too many unsuccessful retry */
     void *external_reference;
 
     time_t expire_time;
@@ -241,7 +251,9 @@ extern "C" {
 
     int r_id;
 
-    int r_reg_period;           /* delay between registration */
+    int r_reg_period;           /* delay between registration (modified by server) */
+    int r_reg_expire;           /* delay between registration (requested by client) */
+
     char *r_aor;                /* sip identity */
     char *r_registrar;          /* registrar */
     char *r_contact;            /* list of contacts string */
@@ -250,7 +262,7 @@ extern "C" {
     char r_qvalue[16];          /* the q value used for routing */
 
     osip_transaction_t *r_last_tr;
-    int r_retry;                /* avoid too many unsuccessfull retry */
+    int r_retry;                /* avoid too many unsuccessful retry */
 
     struct __eXosip_sockaddr addr;
     int len;
@@ -272,7 +284,7 @@ extern "C" {
     int s_reg_period;
     eXosip_dialog_t *s_dialogs;
 
-    int s_retry;                /* avoid too many unsuccessfull retry */
+    int s_retry;                /* avoid too many unsuccessful retry */
     osip_transaction_t *s_inc_tr;
     osip_transaction_t *s_out_tr;
 
@@ -334,8 +346,6 @@ extern "C" {
     jauthinfo_t *next;
   };
 
-  int
-    _eXosip_create_authorization_header (osip_www_authenticate_t * wa, const char *rquri, const char *username, const char *passwd, const char *ha1, osip_authorization_t ** auth, const char *method, const char *pszCNonce, int iNonceCount);
   int _eXosip_create_proxy_authorization_header (osip_proxy_authenticate_t * wa, const char *rquri, const char *username, const char *passwd, const char *ha1, osip_proxy_authorization_t ** auth, const char *method, const char *pszCNonce, int iNonceCount);
   int _eXosip_store_nonce (struct eXosip_t *excontext, const char *call_id, osip_proxy_authenticate_t * wa, int answer_code);
   int _eXosip_delete_nonce (struct eXosip_t *excontext, const char *call_id);
@@ -379,13 +389,14 @@ extern "C" {
   typedef struct eXosip_t eXosip_t;
 
   struct eXosip_t {
-    struct eXtl_protocol *eXtl;
+    struct eXtl_protocol eXtl_transport;
     void *eXtludp_reserved;
     void *eXtltcp_reserved;
 #ifndef DISABLE_TLS
     void *eXtltls_reserved;
     void *eXtldtls_reserved;
 #endif
+    void *tunnel_handle;
     char transport[10];
     char *user_agent;
 
@@ -413,19 +424,42 @@ extern "C" {
     jauthinfo_t *authinfos;
 
     int keep_alive;
+    struct timeval mtimer;
     int keep_alive_options;
     int learn_port;
     int use_rport;
     int dns_capabilities;
+    int enable_dns_cache;
     int dscp;
     int register_with_date;
+    int autoanswer_bye;
+    int ipv6_enable;
     char ipv4_for_gateway[256];
     char ipv6_for_gateway[256];
     struct eXosip_dns_cache dns_entries[MAX_EXOSIP_DNS_ENTRY];
     struct eXosip_account_info account_entries[MAX_EXOSIP_ACCOUNT_INFO];
     struct eXosip_http_auth http_auths[MAX_EXOSIP_HTTP_AUTH];
 
+    /* udp pre-config */
+    char udp_firewall_ip[64];
+    char udp_firewall_port[10];
+
+    /* tcp pre-config */
+    char tcp_firewall_ip[64];
+    char tcp_firewall_port[10];
+
+    /* tls pre-config */
+    char tls_firewall_ip[64];
+    char tls_firewall_port[10];
     int tls_verify_client_certificate;
+    eXosip_tls_ctx_t eXosip_tls_ctx_params;
+    char tls_local_cn_name[128];
+    char tls_client_local_cn_name[128];
+
+    /* dtls pre-config */
+    char dtls_firewall_ip[64];
+    char dtls_firewall_port[10];
+
     CbSipCallback cbsipCallback;
   };
 
@@ -451,6 +485,7 @@ extern "C" {
   int _eXosip_request_add_via (struct eXosip_t *excontext, osip_message_t * request, const char *transport, const char *locip);
 
   void _eXosip_mark_all_registrations_expired (struct eXosip_t *excontext);
+  int _eXosip_check_allow_header (eXosip_dialog_t * jd, osip_message_t * message);
 
   int _eXosip_add_authentication_information (struct eXosip_t *excontext, osip_message_t * req, osip_message_t * last_response);
   int _eXosip_reg_find (struct eXosip_t *excontext, eXosip_reg_t ** reg, osip_transaction_t * tr);
@@ -533,7 +568,7 @@ extern "C" {
 
   int _eXosip_handle_incoming_message (struct eXosip_t *excontext, char *buf, size_t len, int socket, char *host, int port, char *received_host, int *rport_port);
 
-  int _eXosip_transport_set_dscp(struct eXosip_t *excontext, int family, int sock);
+  int _eXosip_transport_set_dscp (struct eXosip_t *excontext, int family, int sock);
 
  /**
   * sets the parameters for the TLS context, which is used for encrypted connections
