@@ -15,6 +15,19 @@
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of portions of this program with the
+  OpenSSL library under certain conditions as described in each
+  individual source file, and distribute linked combinations
+  including the two.
+  You must obey the GNU General Public License in all respects
+  for all of the code used other than OpenSSL.  If you modify
+  file(s) with this exception, you may extend this exception to your
+  version of the file(s), but you are not obligated to do so.  If you
+  do not wish to do so, delete this exception statement from your
+  version.  If you delete this exception statement from all source
+  files in the program, then also delete it here.
 */
 
 #include "eXosip2.h"
@@ -210,6 +223,7 @@ eXosip_subscribe_build_refresh_request (struct eXosip_t *excontext, int did, osi
   if (transaction != NULL && transaction->orig_request != NULL) {
     int pos = 0;
     osip_header_t *_header = NULL;
+    osip_call_info_t *_call_info_header = NULL;
 
     pos = osip_message_get_supported (transaction->orig_request, pos, &_header);
     while (pos >= 0 && _header != NULL) {
@@ -225,6 +239,22 @@ eXosip_subscribe_build_refresh_request (struct eXosip_t *excontext, int did, osi
       pos++;
       pos = osip_message_get_supported (transaction->orig_request, pos, &_header);
     }
+
+    pos = 0;
+    pos = osip_message_get_call_info (transaction->orig_request, pos, &_call_info_header);
+    while (pos >= 0 && _call_info_header != NULL) {
+      osip_call_info_t *_header2;
+
+      i = osip_call_info_clone (_call_info_header, &_header2);
+      if (i != 0) {
+        OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "Error in Call-Info header\n"));
+        break;
+      }
+      osip_list_add (&(*sub)->call_infos, _header2, -1);
+      _call_info_header = NULL;
+      pos++;
+      pos = osip_message_get_call_info (transaction->orig_request, pos, &_call_info_header);
+    }
   }
 
   _eXosip_add_authentication_information (excontext, *sub, NULL);
