@@ -15,12 +15,32 @@
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of portions of this program with the
+  OpenSSL library under certain conditions as described in each
+  individual source file, and distribute linked combinations
+  including the two.
+  You must obey the GNU General Public License in all respects
+  for all of the code used other than OpenSSL.  If you modify
+  file(s) with this exception, you may extend this exception to your
+  version of the file(s), but you are not obligated to do so.  If you
+  do not wish to do so, delete this exception statement from your
+  version.  If you delete this exception statement from all source
+  files in the program, then also delete it here.
 */
 
 #include "eXosip2.h"
 #include "eXtransport.h"
 
-#ifdef WIN32
+#if defined(_MSC_VER) && defined(WIN32) && !defined(_WIN32_WCE)
+#define HAVE_MSTCPIP_H
+#if defined(WINAPI_FAMILY) && (WINAPI_FAMILY == WINAPI_FAMILY_PHONE_APP)
+#undef HAVE_MSTCPIP_H
+#endif
+#endif
+
+#ifdef HAVE_MSTCPIP_H
 #include <Mstcpip.h>
 #else
 #ifdef HAVE_FCNTL_H
@@ -100,8 +120,6 @@ struct eXtltcp {
   int tcp_socket;
   struct sockaddr_storage ai_addr;
 
-  char tcp_firewall_ip[64];
-  char tcp_firewall_port[10];
   struct _tcp_stream socket_tab[EXOSIP_MAX_SOCKETS];
 };
 
@@ -115,8 +133,6 @@ tcp_tl_init (struct eXosip_t *excontext)
   reserved->tcp_socket = 0;
   memset (&reserved->ai_addr, 0, sizeof (struct sockaddr_storage));
   memset (&reserved->socket_tab, 0, sizeof (struct _tcp_stream) * EXOSIP_MAX_SOCKETS);
-  memset (reserved->tcp_firewall_ip, 0, sizeof (reserved->tcp_firewall_ip));
-  memset (reserved->tcp_firewall_port, 0, sizeof (reserved->tcp_firewall_port));
 
   excontext->eXtltcp_reserved = reserved;
   return OSIP_SUCCESS;
@@ -149,8 +165,9 @@ tcp_tl_free (struct eXosip_t *excontext)
   struct eXtltcp *reserved = (struct eXtltcp *) excontext->eXtltcp_reserved;
   int pos;
 
-  memset (reserved->tcp_firewall_ip, 0, sizeof (reserved->tcp_firewall_ip));
-  memset (reserved->tcp_firewall_port, 0, sizeof (reserved->tcp_firewall_port));
+  if (reserved == NULL)
+    return OSIP_SUCCESS;
+
   memset (&reserved->ai_addr, 0, sizeof (struct sockaddr_storage));
   if (reserved->tcp_socket > 0)
     closesocket (reserved->tcp_socket);
@@ -175,18 +192,23 @@ tcp_tl_open (struct eXosip_t *excontext)
   struct addrinfo *curinfo;
   int sock = -1;
 
-  if (eXtl_tcp.proto_port < 0)
-    eXtl_tcp.proto_port = 5060;
+  if (reserved == NULL) {
+    OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "wrong state: create transport layer first\n"));
+    return OSIP_WRONG_STATE;
+  }
+
+  if (excontext->eXtl_transport.proto_port < 0)
+    excontext->eXtl_transport.proto_port = 5060;
 
 
-  res = _eXosip_get_addrinfo (excontext, &addrinfo, eXtl_tcp.proto_ifs, eXtl_tcp.proto_port, eXtl_tcp.proto_num);
+  res = _eXosip_get_addrinfo (excontext, &addrinfo, excontext->eXtl_transport.proto_ifs, excontext->eXtl_transport.proto_port, excontext->eXtl_transport.proto_num);
   if (res)
     return -1;
 
   for (curinfo = addrinfo; curinfo; curinfo = curinfo->ai_next) {
     socklen_t len;
 
-    if (curinfo->ai_protocol && curinfo->ai_protocol != eXtl_tcp.proto_num) {
+    if (curinfo->ai_protocol && curinfo->ai_protocol != excontext->eXtl_transport.proto_num) {
       OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO3, NULL, "Skipping protocol %d\n", curinfo->ai_protocol));
       continue;
     }
@@ -210,13 +232,14 @@ tcp_tl_open (struct eXosip_t *excontext)
 
     {
       int valopt = 1;
+
       setsockopt (sock, SOL_SOCKET, SO_REUSEADDR, (void *) &valopt, sizeof (valopt));
     }
 
 #ifndef DISABLE_MAIN_SOCKET
     res = bind (sock, curinfo->ai_addr, curinfo->ai_addrlen);
     if (res < 0) {
-      OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "Cannot bind socket node:%s family:%d %s\n", eXtl_tcp.proto_ifs, curinfo->ai_family, strerror (ex_errno)));
+      OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "Cannot bind socket node:%s family:%d %s\n", excontext->eXtl_transport.proto_ifs, curinfo->ai_family, strerror (ex_errno)));
       closesocket (sock);
       sock = -1;
       continue;
@@ -228,10 +251,10 @@ tcp_tl_open (struct eXosip_t *excontext)
       memcpy (&reserved->ai_addr, curinfo->ai_addr, curinfo->ai_addrlen);
     }
 
-    if (eXtl_tcp.proto_num == IPPROTO_TCP) {
+    if (excontext->eXtl_transport.proto_num == IPPROTO_TCP) {
       res = listen (sock, SOMAXCONN);
       if (res < 0) {
-        OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "Cannot bind socket node:%s family:%d %s\n", eXtl_tcp.proto_ifs, curinfo->ai_family, strerror (ex_errno)));
+        OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "Cannot bind socket node:%s family:%d %s\n", excontext->eXtl_transport.proto_ifs, curinfo->ai_family, strerror (ex_errno)));
         closesocket (sock);
         sock = -1;
         continue;
@@ -245,22 +268,22 @@ tcp_tl_open (struct eXosip_t *excontext)
   _eXosip_freeaddrinfo (addrinfo);
 
   if (sock < 0) {
-    OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "Cannot bind on port: %i\n", eXtl_tcp.proto_port));
+    OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "Cannot bind on port: %i\n", excontext->eXtl_transport.proto_port));
     return -1;
   }
 
   reserved->tcp_socket = sock;
 
-  if (eXtl_tcp.proto_port == 0) {
+  if (excontext->eXtl_transport.proto_port == 0) {
     /* get port number from socket */
-    if (eXtl_tcp.proto_family == AF_INET)
-      eXtl_tcp.proto_port = ntohs (((struct sockaddr_in *) &reserved->ai_addr)->sin_port);
+    if (excontext->eXtl_transport.proto_family == AF_INET)
+      excontext->eXtl_transport.proto_port = ntohs (((struct sockaddr_in *) &reserved->ai_addr)->sin_port);
     else
-      eXtl_tcp.proto_port = ntohs (((struct sockaddr_in6 *) &reserved->ai_addr)->sin6_port);
-    OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO1, NULL, "Binding on port %i!\n", eXtl_tcp.proto_port));
+      excontext->eXtl_transport.proto_port = ntohs (((struct sockaddr_in6 *) &reserved->ai_addr)->sin6_port);
+    OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO1, NULL, "Binding on port %i!\n", excontext->eXtl_transport.proto_port));
   }
 
-  snprintf (reserved->tcp_firewall_port, sizeof (reserved->tcp_firewall_port), "%i", eXtl_tcp.proto_port);
+  snprintf (excontext->tcp_firewall_port, sizeof (excontext->tcp_firewall_port), "%i", excontext->eXtl_transport.proto_port);
   return OSIP_SUCCESS;
 }
 
@@ -269,9 +292,15 @@ tcp_tl_reset (struct eXosip_t *excontext)
 {
   struct eXtltcp *reserved = (struct eXtltcp *) excontext->eXtltcp_reserved;
   int pos;
+
+  if (reserved == NULL) {
+    OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "wrong state: create transport layer first\n"));
+    return OSIP_WRONG_STATE;
+  }
+
   for (pos = 0; pos < EXOSIP_MAX_SOCKETS; pos++) {
-    if (reserved->socket_tab[pos].socket>0)
-      reserved->socket_tab[pos].invalid=1;
+    if (reserved->socket_tab[pos].socket > 0)
+      reserved->socket_tab[pos].invalid = 1;
   }
   return OSIP_SUCCESS;
 }
@@ -282,6 +311,11 @@ tcp_tl_set_fdset (struct eXosip_t *excontext, fd_set * osip_fdset, fd_set * osip
   struct eXtltcp *reserved = (struct eXtltcp *) excontext->eXtltcp_reserved;
   int pos;
 
+  if (reserved == NULL) {
+    OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "wrong state: create transport layer first\n"));
+    return OSIP_WRONG_STATE;
+  }
+
 #ifndef DISABLE_MAIN_SOCKET
   if (reserved->tcp_socket <= 0)
     return -1;
@@ -475,6 +509,11 @@ tcp_tl_read_message (struct eXosip_t *excontext, fd_set * osip_fdset, fd_set * o
   struct eXtltcp *reserved = (struct eXtltcp *) excontext->eXtltcp_reserved;
   int pos = 0;
 
+  if (reserved == NULL) {
+    OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "wrong state: create transport layer first\n"));
+    return OSIP_WRONG_STATE;
+  }
+
   if (FD_ISSET (reserved->tcp_socket, osip_fdset)) {
     /* accept incoming connection */
     char src6host[NI_MAXHOST];
@@ -485,7 +524,7 @@ tcp_tl_read_message (struct eXosip_t *excontext, fd_set * osip_fdset, fd_set * o
 
     socklen_t slen;
 
-    if (eXtl_tcp.proto_family == AF_INET)
+    if (excontext->eXtl_transport.proto_family == AF_INET)
       slen = sizeof (struct sockaddr_in);
     else
       slen = sizeof (struct sockaddr_in6);
@@ -516,33 +555,34 @@ tcp_tl_read_message (struct eXosip_t *excontext, fd_set * osip_fdset, fd_set * o
         memset (&reserved->ai_addr, 0, sizeof (struct sockaddr_storage));
         if (reserved->tcp_socket > 0) {
           closesocket (reserved->tcp_socket);
-	  for (i = 0; i < EXOSIP_MAX_SOCKETS; i++) {
-	    if (reserved->socket_tab[i].socket > 0 && reserved->socket_tab[i].is_server > 0)
-	      _tcp_tl_close_sockinfo (&reserved->socket_tab[i]);
-	  }
-	}
+          for (i = 0; i < EXOSIP_MAX_SOCKETS; i++) {
+            if (reserved->socket_tab[i].socket > 0 && reserved->socket_tab[i].is_server > 0)
+              _tcp_tl_close_sockinfo (&reserved->socket_tab[i]);
+          }
+        }
         tcp_tl_open (excontext);
       }
 #endif
     }
     else {
       reserved->socket_tab[pos].socket = sock;
-      reserved->socket_tab[pos].is_server=1;
+      reserved->socket_tab[pos].is_server = 1;
       OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO1, NULL, "New TCP connection accepted\n"));
 
       {
-	int valopt = 1;
-	setsockopt (sock, SOL_SOCKET, SO_REUSEADDR, (void *) &valopt, sizeof (valopt));
+        int valopt = 1;
+
+        setsockopt (sock, SOL_SOCKET, SO_REUSEADDR, (void *) &valopt, sizeof (valopt));
       }
 
       memset (src6host, 0, sizeof (src6host));
 
-      if (eXtl_tcp.proto_family == AF_INET)
+      if (excontext->eXtl_transport.proto_family == AF_INET)
         recvport = ntohs (((struct sockaddr_in *) &sa)->sin_port);
       else
         recvport = ntohs (((struct sockaddr_in6 *) &sa)->sin6_port);
 
-      _eXosip_transport_set_dscp(excontext, eXtl_tcp.proto_family, sock);
+      _eXosip_transport_set_dscp (excontext, excontext->eXtl_transport.proto_family, sock);
 
 #if defined(__arc__)
       {
@@ -670,77 +710,39 @@ _tcp_tl_check_connected (struct eXosip_t *excontext)
 
   for (pos = 0; pos < EXOSIP_MAX_SOCKETS; pos++) {
     if (reserved->socket_tab[pos].invalid > 0) {
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_INFO2, NULL,
-                       "_tcp_tl_check_connected: socket node is in invalid state:%s:%i, socket %d [pos=%d], family:%d\n",
-                       reserved->socket_tab[pos].remote_ip, reserved->socket_tab[pos].remote_port, reserved->socket_tab[pos].socket, pos, reserved->socket_tab[pos].ai_addr.sa_family));
-          _tcp_tl_close_sockinfo (&reserved->socket_tab[pos]);
-          continue;
+      OSIP_TRACE (osip_trace
+                  (__FILE__, __LINE__, OSIP_INFO2, NULL,
+                   "_tcp_tl_check_connected: socket node is in invalid state:%s:%i, socket %d [pos=%d], family:%d\n",
+                   reserved->socket_tab[pos].remote_ip, reserved->socket_tab[pos].remote_port, reserved->socket_tab[pos].socket, pos, reserved->socket_tab[pos].ai_addr.sa_family));
+      _tcp_tl_close_sockinfo (&reserved->socket_tab[pos]);
+      continue;
     }
 
     if (reserved->socket_tab[pos].socket > 0 && reserved->socket_tab[pos].ai_addrlen > 0) {
-      res = connect (reserved->socket_tab[pos].socket, &reserved->socket_tab[pos].ai_addr, reserved->socket_tab[pos].ai_addrlen);
-      if (res < 0) {
-        int status = ex_errno;
-
-#if defined(_WIN32_WCE) || defined(WIN32)
-        if (status == WSAEISCONN) {
-          reserved->socket_tab[pos].ai_addrlen = 0;     /* already connected */
-          continue;
-        }
-#else
-        if (status == EISCONN) {
-          reserved->socket_tab[pos].ai_addrlen = 0;     /* already connected */
-          continue;
-        }
-#endif
-#if defined(_WIN32_WCE) || defined(WIN32)
-        if (status != WSAEWOULDBLOCK && status != WSAEALREADY && status != WSAEINVAL) {
-#else
-        if (status != EINPROGRESS && status != EALREADY) {
-#endif
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_INFO2, NULL,
-                       "_tcp_tl_check_connected: Cannot connect socket node:%s:%i, socket %d [pos=%d], family:%d, %s[%d]\n",
-                       reserved->socket_tab[pos].remote_ip, reserved->socket_tab[pos].remote_port, reserved->socket_tab[pos].socket, pos, reserved->socket_tab[pos].ai_addr.sa_family, strerror (status), status));
-          _tcp_tl_close_sockinfo (&reserved->socket_tab[pos]);
-          continue;
-        }
-        else {
-          res = _tcp_tl_is_connected (reserved->socket_tab[pos].socket);
-          if (res > 0) {
-            OSIP_TRACE (osip_trace
-                        (__FILE__, __LINE__, OSIP_INFO2, NULL,
-                         "_tcp_tl_check_connected: socket node:%s:%i, socket %d [pos=%d], family:%d, in progress\n",
-                         reserved->socket_tab[pos].remote_ip, reserved->socket_tab[pos].remote_port, reserved->socket_tab[pos].socket, pos, reserved->socket_tab[pos].ai_addr.sa_family));
-            continue;
-          }
-          else if (res == 0) {
-            OSIP_TRACE (osip_trace
-                        (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                         "_tcp_tl_check_connected: socket node:%s:%i , socket %d [pos=%d], family:%d, connected\n",
-                         reserved->socket_tab[pos].remote_ip, reserved->socket_tab[pos].remote_port, reserved->socket_tab[pos].socket, pos, reserved->socket_tab[pos].ai_addr.sa_family));
-            /* stop calling "connect()" */
-            reserved->socket_tab[pos].ai_addrlen = 0;
-            continue;
-          }
-          else {
-            OSIP_TRACE (osip_trace
-                        (__FILE__, __LINE__, OSIP_INFO2, NULL,
-                         "_tcp_tl_check_connected: socket node:%s:%i, socket %d [pos=%d], family:%d, error\n",
-                         reserved->socket_tab[pos].remote_ip, reserved->socket_tab[pos].remote_port, reserved->socket_tab[pos].socket, pos, reserved->socket_tab[pos].ai_addr.sa_family));
-            _tcp_tl_close_sockinfo (&reserved->socket_tab[pos]);
-            continue;
-          }
-        }
+      res = _tcp_tl_is_connected (reserved->socket_tab[pos].socket);
+      if (res > 0) {
+        OSIP_TRACE (osip_trace
+                    (__FILE__, __LINE__, OSIP_INFO2, NULL,
+                     "_tcp_tl_check_connected: socket node:%s:%i, socket %d [pos=%d], family:%d, in progress\n",
+                     reserved->socket_tab[pos].remote_ip, reserved->socket_tab[pos].remote_port, reserved->socket_tab[pos].socket, pos, reserved->socket_tab[pos].ai_addr.sa_family));
+        continue;
       }
-      else {
+      else if (res == 0) {
         OSIP_TRACE (osip_trace
                     (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                     "_tcp_tl_check_connected: socket node:%s:%i , socket %d [pos=%d], family:%d, connected (with connect)\n",
+                     "_tcp_tl_check_connected: socket node:%s:%i , socket %d [pos=%d], family:%d, connected\n",
                      reserved->socket_tab[pos].remote_ip, reserved->socket_tab[pos].remote_port, reserved->socket_tab[pos].socket, pos, reserved->socket_tab[pos].ai_addr.sa_family));
         /* stop calling "connect()" */
         reserved->socket_tab[pos].ai_addrlen = 0;
+        continue;
+      }
+      else {
+        OSIP_TRACE (osip_trace
+                    (__FILE__, __LINE__, OSIP_INFO2, NULL,
+                     "_tcp_tl_check_connected: socket node:%s:%i, socket %d [pos=%d], family:%d, error\n",
+                     reserved->socket_tab[pos].remote_ip, reserved->socket_tab[pos].remote_port, reserved->socket_tab[pos].socket, pos, reserved->socket_tab[pos].ai_addr.sa_family));
+        _tcp_tl_close_sockinfo (&reserved->socket_tab[pos]);
+        continue;
       }
     }
   }
@@ -792,10 +794,13 @@ _tcp_tl_connect_socket (struct eXosip_t *excontext, char *host, int port)
 
   for (curinfo = addrinfo; curinfo; curinfo = curinfo->ai_next) {
     int i;
-    if (curinfo->ai_protocol && curinfo->ai_protocol != IPPROTO_TCP) continue;
+
+    if (curinfo->ai_protocol && curinfo->ai_protocol != IPPROTO_TCP)
+      continue;
 
     res = getnameinfo ((struct sockaddr *) curinfo->ai_addr, curinfo->ai_addrlen, src6host, NI_MAXHOST, NULL, 0, NI_NUMERICHOST);
-    if (res != 0) continue;
+    if (res != 0)
+      continue;
 
     i = _tcp_tl_find_socket (excontext, src6host, port);
     if (i >= 0) {
@@ -813,7 +818,7 @@ _tcp_tl_connect_socket (struct eXosip_t *excontext, char *host, int port)
     res = getnameinfo ((struct sockaddr *) curinfo->ai_addr, curinfo->ai_addrlen, src6host, NI_MAXHOST, NULL, 0, NI_NUMERICHOST);
 
     if (res == 0) {
-      OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO2, NULL, "New binding with %s:%i\n", src6host, port));
+      OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO1, NULL, "New binding with %s:%i\n", src6host, port));
     }
 
     sock = (int) socket (curinfo->ai_family, curinfo->ai_socktype, curinfo->ai_protocol);
@@ -849,7 +854,7 @@ _tcp_tl_connect_socket (struct eXosip_t *excontext, char *host, int port)
         continue;
       }
     }
-#if !defined(_WIN32_WCE) && defined(_MSC_VER)
+#ifdef HAVE_MSTCPIP_H
     {
       DWORD err = 0L;
       DWORD dwBytes = 0L;
@@ -898,16 +903,21 @@ _tcp_tl_connect_socket (struct eXosip_t *excontext, char *host, int port)
       setsockopt (sock, SOL_SOCKET, SO_NOSIGPIPE, (void *) &val, sizeof (int));
 #endif
 
+    }
+#endif
+
 #if TCP_NODELAY
+    {
+      int val;
+
       val = 1;
       if (setsockopt (sock, IPPROTO_TCP, TCP_NODELAY, (char *) &val, sizeof (int)) != 0) {
         OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO2, NULL, "Cannot set socket flag (TCP_NODELAY)\n"));
       }
-#endif
     }
 #endif
 
-    _eXosip_transport_set_dscp(excontext, eXtl_tcp.proto_family, sock);
+    _eXosip_transport_set_dscp (excontext, excontext->eXtl_transport.proto_family, sock);
 
     OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO2, NULL, "socket node:%s , socket %d, family:%d set to non blocking mode\n", host, sock, curinfo->ai_family));
     res = connect (sock, curinfo->ai_addr, curinfo->ai_addrlen);
@@ -946,6 +956,8 @@ _tcp_tl_connect_socket (struct eXosip_t *excontext, char *host, int port)
           CFWriteStreamOpen (reserved->socket_tab[pos].writeStream);
 #endif
           OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO1, NULL, "socket node:%s , socket %d [pos=%d], family:%d, connected\n", host, sock, pos, curinfo->ai_family));
+          selected_ai_addrlen = 0;
+          memcpy (&selected_ai_addr, curinfo->ai_addr, sizeof (struct sockaddr));
           break;
         }
         else {
@@ -1079,8 +1091,9 @@ _tcp_tl_update_local_target_use_ephemeral_port (struct eXosip_t *excontext, osip
       if (ephemeral_port > 0) {
         if (co->url->port)
           osip_free (co->url->port);
-        co->url->port = osip_malloc (10);
+        co->url->port = (char *) osip_malloc (10);
         snprintf (co->url->port, 9, "%i", ephemeral_port);
+        osip_message_force_update (req);
       }
     }
   }
@@ -1104,12 +1117,14 @@ _tcp_tl_update_local_target (struct eXosip_t *excontext, osip_message_t * req, c
         if (natted_port > 0) {
           if (co->url->port)
             osip_free (co->url->port);
-          co->url->port = osip_malloc (10);
+          co->url->port = (char *) osip_malloc (10);
           snprintf (co->url->port, 9, "%i", natted_port);
+          osip_message_force_update (req);
         }
         if (natted_ip != NULL && natted_ip[0] != '\0') {
           osip_free (co->url->host);
           co->url->host = osip_strdup (natted_ip);
+          osip_message_force_update (req);
         }
       }
     }
@@ -1128,6 +1143,11 @@ tcp_tl_send_message (struct eXosip_t *excontext, osip_transaction_t * tr, osip_m
   int pos = -1;
   osip_naptr_t *naptr_record = NULL;
 
+  if (reserved == NULL) {
+    OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "wrong state: create transport layer first\n"));
+    return OSIP_WRONG_STATE;
+  }
+
   if (host == NULL) {
     host = sip->req_uri->host;
     if (sip->req_uri->port != NULL)
@@ -1244,7 +1264,7 @@ tcp_tl_send_message (struct eXosip_t *excontext, osip_transaction_t * tr, osip_m
         if (reserved->socket_tab[pos].socket == out_socket) {
           OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO1, NULL, "reusing REQUEST connection (to dest=%s:%i)\n", reserved->socket_tab[pos].remote_ip, reserved->socket_tab[pos].remote_port));
           _tcp_tl_update_local_target_use_ephemeral_port (excontext, sip, reserved->socket_tab[pos].ephemeral_port);
-          if (reserved->tcp_firewall_ip[0] != '\0')
+          if (excontext->tcp_firewall_ip[0] != '\0')
             _tcp_tl_update_local_target (excontext, sip, reserved->socket_tab[pos].natted_ip, reserved->socket_tab[pos].natted_port);
           break;
         }
@@ -1286,7 +1306,7 @@ tcp_tl_send_message (struct eXosip_t *excontext, osip_transaction_t * tr, osip_m
     if (pos >= 0) {
       out_socket = reserved->socket_tab[pos].socket;
       _tcp_tl_update_local_target_use_ephemeral_port (excontext, sip, reserved->socket_tab[pos].ephemeral_port);
-      if (reserved->tcp_firewall_ip[0] != '\0')
+      if (excontext->tcp_firewall_ip[0] != '\0')
         _tcp_tl_update_local_target (excontext, sip, reserved->socket_tab[pos].natted_ip, reserved->socket_tab[pos].natted_port);
     }
   }
@@ -1366,6 +1386,18 @@ tcp_tl_send_message (struct eXosip_t *excontext, osip_transaction_t * tr, osip_m
   }
 
   OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO1, NULL, "Message sent: (to dest=%s:%i) \n%s\n", host, port, message));
+
+  if (pos >= 0 && excontext->enable_dns_cache == 1 && osip_strcasecmp (host, reserved->socket_tab[pos].remote_ip) != 0 && MSG_IS_REQUEST (sip)) {
+    if (MSG_IS_REGISTER (sip)) {
+      struct eXosip_dns_cache entry;
+
+      memset (&entry, 0, sizeof (struct eXosip_dns_cache));
+      snprintf (entry.host, sizeof (entry.host), "%s", host);
+      snprintf (entry.ip, sizeof (entry.ip), "%s", reserved->socket_tab[pos].remote_ip);
+      eXosip_set_option (excontext, EXOSIP_OPT_ADD_DNS_CACHE, (void *) &entry);
+    }
+  }
+
   i = _tcp_tl_send (excontext, out_socket, (const void *) message, (int) length);
   osip_free (message);
   return i;
@@ -1408,6 +1440,11 @@ tcp_tl_keepalive (struct eXosip_t *excontext)
   int pos;
   int i;
 
+  if (reserved == NULL) {
+    OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "wrong state: create transport layer first\n"));
+    return OSIP_WRONG_STATE;
+  }
+
   if (reserved->tcp_socket <= 0)
     return OSIP_UNDEFINED_ERROR;
 
@@ -1492,6 +1529,11 @@ tcp_tl_set_socket (struct eXosip_t *excontext, int socket)
 {
   struct eXtltcp *reserved = (struct eXtltcp *) excontext->eXtltcp_reserved;
 
+  if (reserved == NULL) {
+    OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "wrong state: create transport layer first\n"));
+    return OSIP_WRONG_STATE;
+  }
+
   reserved->tcp_socket = socket;
 
   return OSIP_SUCCESS;
@@ -1500,18 +1542,16 @@ tcp_tl_set_socket (struct eXosip_t *excontext, int socket)
 static int
 tcp_tl_masquerade_contact (struct eXosip_t *excontext, const char *public_address, int port)
 {
-  struct eXtltcp *reserved = (struct eXtltcp *) excontext->eXtltcp_reserved;
-
   if (public_address == NULL || public_address[0] == '\0') {
-    memset (reserved->tcp_firewall_ip, '\0', sizeof (reserved->tcp_firewall_ip));
-    memset (reserved->tcp_firewall_port, '\0', sizeof (reserved->tcp_firewall_port));
-    if (eXtl_tcp.proto_port > 0)
-      snprintf (reserved->tcp_firewall_port, sizeof (reserved->tcp_firewall_port), "%i", eXtl_tcp.proto_port);
+    memset (excontext->tcp_firewall_ip, '\0', sizeof (excontext->tcp_firewall_ip));
+    memset (excontext->tcp_firewall_port, '\0', sizeof (excontext->tcp_firewall_port));
+    if (excontext->eXtl_transport.proto_port > 0)
+      snprintf (excontext->tcp_firewall_port, sizeof (excontext->tcp_firewall_port), "%i", excontext->eXtl_transport.proto_port);
     return OSIP_SUCCESS;
   }
-  snprintf (reserved->tcp_firewall_ip, sizeof (reserved->tcp_firewall_ip), "%s", public_address);
+  snprintf (excontext->tcp_firewall_ip, sizeof (excontext->tcp_firewall_ip), "%s", public_address);
   if (port > 0) {
-    snprintf (reserved->tcp_firewall_port, sizeof (reserved->tcp_firewall_port), "%i", port);
+    snprintf (excontext->tcp_firewall_port, sizeof (excontext->tcp_firewall_port), "%i", port);
   }
   return OSIP_SUCCESS;
 }
@@ -1524,11 +1564,16 @@ tcp_tl_get_masquerade_contact (struct eXosip_t *excontext, char *ip, int ip_size
   memset (ip, 0, ip_size);
   memset (port, 0, port_size);
 
-  if (reserved->tcp_firewall_ip[0] != '\0')
-    snprintf (ip, ip_size, "%s", reserved->tcp_firewall_ip);
+  if (reserved == NULL) {
+    OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "wrong state: create transport layer first\n"));
+    return OSIP_WRONG_STATE;
+  }
 
-  if (reserved->tcp_firewall_port[0] != '\0')
-    snprintf (port, port_size, "%s", reserved->tcp_firewall_port);
+  if (excontext->tcp_firewall_ip[0] != '\0')
+    snprintf (ip, ip_size, "%s", excontext->tcp_firewall_ip);
+
+  if (excontext->tcp_firewall_port[0] != '\0')
+    snprintf (port, port_size, "%s", excontext->tcp_firewall_port);
   return OSIP_SUCCESS;
 }
 
@@ -1554,3 +1599,9 @@ struct eXtl_protocol eXtl_tcp = {
   &tcp_tl_get_masquerade_contact,
   &tcp_tl_reset
 };
+
+void
+eXosip_transport_tcp_init (struct eXosip_t *excontext)
+{
+  memcpy (&excontext->eXtl_transport, &eXtl_tcp, sizeof (struct eXtl_protocol));
+}
