@@ -15,6 +15,19 @@
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of portions of this program with the
+  OpenSSL library under certain conditions as described in each
+  individual source file, and distribute linked combinations
+  including the two.
+  You must obey the GNU General Public License in all respects
+  for all of the code used other than OpenSSL.  If you modify
+  file(s) with this exception, you may extend this exception to your
+  version of the file(s), but you are not obligated to do so.  If you
+  do not wish to do so, delete this exception statement from your
+  version.  If you delete this exception statement from all source
+  files in the program, then also delete it here.
 */
 
 #ifndef __EXTRANSPORT_H__
@@ -59,13 +72,10 @@ struct eXtl_protocol {
   int (*tl_reset) (struct eXosip_t * excontext);
 };
 
-extern struct eXtl_protocol eXtl_udp;
-extern struct eXtl_protocol eXtl_tcp;
-
-#ifndef DISABLE_TLS
-extern struct eXtl_protocol eXtl_tls;
-extern struct eXtl_protocol eXtl_dtls;
-#endif
+void eXosip_transport_udp_init (struct eXosip_t *excontext);
+void eXosip_transport_tcp_init (struct eXosip_t *excontext);
+void eXosip_transport_tls_init (struct eXosip_t *excontext);
+void eXosip_transport_dtls_init (struct eXosip_t *excontext);
 
 #if defined (WIN32) || defined (_WIN32_WCE)
 #define eXFD_SET(A, B)   FD_SET((unsigned int) A, B)
