@@ -15,6 +15,19 @@
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of portions of this program with the
+  OpenSSL library under certain conditions as described in each
+  individual source file, and distribute linked combinations
+  including the two.
+  You must obey the GNU General Public License in all respects
+  for all of the code used other than OpenSSL.  If you modify
+  file(s) with this exception, you may extend this exception to your
+  version of the file(s), but you are not obligated to do so.  If you
+  do not wish to do so, delete this exception statement from your
+  version.  If you delete this exception statement from all source
+  files in the program, then also delete it here.
 */
 
 #include "eXosip2.h"
@@ -585,237 +598,6 @@ done:
   }
 }
 
-int
-_eXosip_create_authorization_header (osip_www_authenticate_t * wa, const char *rquri, const char *username, const char *passwd, const char *ha1, osip_authorization_t ** auth, const char *method, const char *pCNonce, int iNonceCount)
-{
-  osip_authorization_t *aut;
-
-  char *qop = NULL;
-  char *Alg = "MD5";
-  int version = 0;
-  int i;
-
-  /* make some test */
-  if (passwd == NULL)
-    return OSIP_BADPARAMETER;
-  if (wa == NULL)
-    return OSIP_BADPARAMETER;
-
-  if (wa->auth_type == NULL || (wa->nonce == NULL)) {
-    OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "www_authenticate header is not acceptable.\n"));
-    return OSIP_SYNTAXERROR;
-  }
-
-  if (wa->realm == NULL) {
-    OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "www_authenticate header contains an empty realm: contact your admin!\n"));
-  }
-
-  if (0 != osip_strcasecmp ("Digest", wa->auth_type)) {
-    OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "Authentication method not supported. (Digest only).\n"));
-    return OSIP_UNDEFINED_ERROR;
-  }
-  /* "MD5" is invalid, but some servers use it. */
-  if (wa->algorithm != NULL) {
-    if (0 == osip_strcasecmp ("MD5", wa->algorithm)
-        || 0 == osip_strcasecmp ("\"MD5\"", wa->algorithm)) {
-    }
-    else if (0 == osip_strcasecmp ("AKAv1-MD5", wa->algorithm)
-             || 0 == osip_strcasecmp ("\"AKAv1-MD5\"", wa->algorithm)) {
-      Alg = "AKAv1-MD5";
-    }
-    else if (0 == osip_strcasecmp ("AKAv2-MD5", wa->algorithm)
-             || 0 == osip_strcasecmp ("\"AKAv2-MD5\"", wa->algorithm)) {
-      Alg = "AKAv2-MD5";
-    }
-    else {
-      OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "Authentication method not supported. (MD5, AKAv1-MD5, AKAv2-MD5)\n"));
-      return OSIP_UNDEFINED_ERROR;
-    }
-  }
-
-  i = osip_authorization_init (&aut);
-  if (i != 0) {
-    OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "allocation with authorization_init failed.\n"));
-    return i;
-  }
-
-  /* just copy some feilds from response to new request */
-  osip_authorization_set_auth_type (aut, osip_strdup ("Digest"));
-  osip_authorization_set_realm (aut, osip_strdup (osip_www_authenticate_get_realm (wa)));
-  if (aut->realm == NULL) {
-    aut->realm = (char *) osip_malloc (3);
-    aut->realm[0] = '"';
-    aut->realm[1] = '"';
-    aut->realm[2] = '\0';
-  }
-  osip_authorization_set_nonce (aut, osip_strdup (osip_www_authenticate_get_nonce (wa)));
-  if (osip_www_authenticate_get_opaque (wa) != NULL)
-    osip_authorization_set_opaque (aut, osip_strdup (osip_www_authenticate_get_opaque (wa)));
-  /* copy the username field in new request */
-  aut->username = osip_malloc (strlen (username) + 3);
-  if (aut->username == NULL) {
-    osip_authorization_free (aut);
-    return OSIP_NOMEM;
-  }
-  sprintf (aut->username, "\"%s\"", username);
-
-  {
-    char *tmp = osip_malloc (strlen (rquri) + 3);
-
-    if (tmp == NULL) {
-      osip_authorization_free (aut);
-      return OSIP_NOMEM;
-    }
-    sprintf (tmp, "\"%s\"", rquri);
-    osip_authorization_set_uri (aut, tmp);
-  }
-
-  osip_authorization_set_algorithm (aut, osip_strdup (Alg));
-
-  qop = osip_www_authenticate_get_qop_options (wa);
-  if (qop == NULL || qop[0] == '\0' || strlen (qop) < 4)
-    qop = NULL;
-
-
-  {
-    char *pszNonce = osip_strdup_without_quote (osip_www_authenticate_get_nonce (wa));
-    char *pszCNonce = NULL;
-    const char *pszUser = username;
-    char *pszRealm = NULL;
-    const char *pszPass = NULL;
-    char *szNonceCount = NULL;
-    const char *pszMethod = method;     /* previous_answer->cseq->method; */
-    char *pszQop = NULL;
-    const char *pszURI = rquri;
-
-    HASHHEX HA1;
-    HASHHEX HA2 = "";
-    HASHHEX Response;
-    RESHEXAKA2 Response2;
-    const char *pha1 = NULL;
-
-    if (osip_authorization_get_realm (aut) == NULL) {
-      pszRealm = osip_strdup ("");
-    }
-    else {
-      pszRealm = osip_strdup_without_quote (osip_authorization_get_realm (aut));
-    }
-
-    if (qop != NULL) {
-      /* only accept qop="auth" */
-      pszQop = osip_strdup ("auth");
-      if (pszQop == NULL) {
-        osip_authorization_free (aut);
-        osip_free (pszNonce);
-        osip_free (pszRealm);
-        return OSIP_NOMEM;
-      }
-
-      szNonceCount = osip_malloc (10);
-      if (szNonceCount == NULL) {
-        osip_authorization_free (aut);
-        osip_free (pszNonce);
-        osip_free (pszRealm);
-        osip_free (pszQop);
-        return OSIP_NOMEM;
-      }
-      snprintf (szNonceCount, 9, "%.8i", iNonceCount);
-
-      pszCNonce = osip_strdup (pCNonce);
-      if (pszCNonce == NULL) {
-        osip_authorization_free (aut);
-        osip_free (pszNonce);
-        osip_free (pszRealm);
-        osip_free (pszQop);
-        osip_free (szNonceCount);
-        return OSIP_NOMEM;
-      }
-
-      osip_authorization_set_message_qop (aut, osip_strdup ("auth"));
-      osip_authorization_set_nonce_count (aut, osip_strdup (szNonceCount));
-
-      {
-        char *tmp = osip_malloc (strlen (pszCNonce) + 3);
-
-        if (tmp == NULL) {
-          osip_authorization_free (aut);
-          osip_free (pszNonce);
-          osip_free (pszCNonce);
-          osip_free (pszRealm);
-          osip_free (pszQop);
-          osip_free (szNonceCount);
-          return OSIP_NOMEM;
-        }
-        sprintf (tmp, "\"%s\"", pszCNonce);
-        osip_authorization_set_cnonce (aut, tmp);
-      }
-    }
-
-    pszPass = passwd;
-
-    /* Depending on which algorithm the response will be calculated, MD5 or AKAv1-MD5 */
-    if (0 == osip_strcasecmp (Alg, "MD5")) {
-      if (ha1 && ha1[0]) {
-        /* Depending on algorithm=md5 */
-        pha1 = ha1;
-      }
-      else {
-        DigestCalcHA1 ("MD5", pszUser, pszRealm, pszPass, pszNonce, pszCNonce, HA1);
-        pha1 = HA1;
-      }
-
-      version = 0;
-      DigestCalcResponse ((char *) pha1, pszNonce, szNonceCount, pszCNonce, pszQop, version, pszMethod, pszURI, HA2, Response);
-    }
-    else {
-      if (0 == osip_strcasecmp (Alg, "AKAv1-MD5"))
-        version = 1;
-      else
-        version = 2;
-
-      DigestCalcResponseAka (pszPass, pszNonce, pszCNonce, pszQop, pszMethod, pszURI, version, Response2);
-      if (ha1 && ha1[0]) {
-        /* Depending on algorithm=md5 */
-        pha1 = ha1;
-      }
-      else {
-        DigestCalcHA1 ("MD5", pszUser, pszRealm, Response2, pszNonce, pszCNonce, HA1);
-        pha1 = HA1;
-      }
-
-      DigestCalcResponse ((char *) pha1, pszNonce, szNonceCount, pszCNonce, pszQop, version, pszMethod, pszURI, HA2, Response);
-
-    }
-
-    OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO4, NULL, "Response in authorization |%s|\n", Response));
-    {
-      char *resp = osip_malloc (35);
-
-      if (resp == NULL) {
-        osip_authorization_free (aut);
-        osip_free (pszNonce);
-        osip_free (pszCNonce);
-        osip_free (pszRealm);
-        osip_free (pszQop);
-        osip_free (szNonceCount);
-        return OSIP_NOMEM;
-      }
-
-      snprintf (resp, 35, "\"%s\"", Response);
-      resp[34] = 0;
-      osip_authorization_set_response (aut, resp);
-    }
-    osip_free (pszNonce);
-    osip_free (pszCNonce);
-    osip_free (pszRealm);
-    osip_free (pszQop);
-    osip_free (szNonceCount);
-  }
-
-  *auth = aut;
-  return OSIP_SUCCESS;
-}
-
 int
 _eXosip_create_proxy_authorization_header (osip_proxy_authenticate_t * wa, const char *rquri, const char *username, const char *passwd, const char *ha1, osip_proxy_authorization_t ** auth, const char *method, const char *pCNonce, int iNonceCount)
 {
