@@ -15,6 +15,19 @@
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of portions of this program with the
+  OpenSSL library under certain conditions as described in each
+  individual source file, and distribute linked combinations
+  including the two.
+  You must obey the GNU General Public License in all respects
+  for all of the code used other than OpenSSL.  If you modify
+  file(s) with this exception, you may extend this exception to your
+  version of the file(s), but you are not obligated to do so.  If you
+  do not wish to do so, delete this exception statement from your
+  version.  If you delete this exception statement from all source
+  files in the program, then also delete it here.
 */
 
 
@@ -24,7 +37,6 @@
 #include <winsock2.h>
 #include "inet_ntop.h"
 #elif WIN32
-#include <windowsx.h>
 #include <winsock2.h>
 #include <ws2tcpip.h>
 #include "inet_ntop.h"
@@ -179,23 +191,7 @@ _eXosip_snd_message (struct eXosip_t *excontext, osip_transaction_t * tr, osip_m
   }
 
   i = -1;
-  if (osip_strcasecmp (via->protocol, "udp") == 0) {
-    i = eXtl_udp.tl_send_message (excontext, tr, sip, host, port, out_socket);
-  }
-  else if (osip_strcasecmp (via->protocol, "tcp") == 0) {
-    i = eXtl_tcp.tl_send_message (excontext, tr, sip, host, port, out_socket);
-  }
-#ifdef HAVE_OPENSSL_SSL_H
-  else if (osip_strcasecmp (via->protocol, "tls") == 0) {
-    i = eXtl_tls.tl_send_message (excontext, tr, sip, host, port, out_socket);
-  }
-  else if (osip_strcasecmp (via->protocol, "dtls-udp") == 0) {
-    i = -1;
-#if !(OPENSSL_VERSION_NUMBER < 0x00908000L)
-    i = eXtl_dtls.tl_send_message (excontext, tr, sip, host, port, out_socket);
-#endif
-  }
-#endif
+  i = excontext->eXtl_transport.tl_send_message (excontext, tr, sip, host, port, out_socket);
   if (i != 0) {
     return i;
   }
@@ -384,7 +380,11 @@ cb_rcvrequest (int type, osip_transaction_t * tr, osip_message_t * sip)
     OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO3, NULL, "cb_rcv? (id=%i)\r\n", tr->transactionid));
 
     if (MSG_IS_BYE (sip)) {
-      /* already sent */
+      if (excontext->autoanswer_bye == 0) {
+        /* not already sent */
+        _eXosip_report_call_event (excontext, EXOSIP_CALL_MESSAGE_NEW, jc, jd, tr);
+        _eXosip_report_call_event (excontext, EXOSIP_CALL_CLOSED, jc, jd, tr);
+      }
     }
     else
       _eXosip_report_call_event (excontext, EXOSIP_CALL_MESSAGE_NEW, jc, jd, tr);
@@ -628,10 +628,12 @@ cb_rcv1xx (int type, osip_transaction_t * tr, osip_message_t * sip)
 
     if (jd != NULL && MSG_IS_RESPONSE_FOR (sip, "INVITE")
         && sip->status_code < 180) {
+      _eXosip_check_allow_header (jd, sip);
       _eXosip_report_call_event (excontext, EXOSIP_CALL_PROCEEDING, jc, jd, tr);
     }
     else if (jd != NULL && MSG_IS_RESPONSE_FOR (sip, "INVITE")
              && sip->status_code >= 180) {
+      _eXosip_check_allow_header (jd, sip);
       _eXosip_report_call_event (excontext, EXOSIP_CALL_RINGING, jc, jd, tr);
     }
 #ifndef MINISIZE
@@ -735,6 +737,8 @@ cb_rcv2xx_4invite (osip_transaction_t * tr, osip_message_t * sip)
     osip_header_t *se_exp = NULL;
     osip_header_t *se_exp_answer = NULL;
 
+    _eXosip_check_allow_header (jd, sip);
+
     osip_message_header_get_byname (tr->orig_request, "session-expires", 0, &se_exp);
     if (se_exp == NULL)
       osip_message_header_get_byname (tr->orig_request, "x", 0, &se_exp);
@@ -1016,6 +1020,10 @@ cb_rcv2xx (int type, osip_transaction_t * tr, osip_message_t * sip)
       _eXosip_report_call_event (excontext, EXOSIP_CALL_MESSAGE_ANSWERED, jc, jd, tr);
       return;
     }
+
+    if (jd != NULL)
+      _eXosip_check_allow_header (jd, sip);
+
     if (MSG_IS_RESPONSE_FOR (sip, "UPDATE")) {
       if (jd != NULL) {
         osip_header_t *se_exp = NULL;
