@@ -15,6 +15,19 @@
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of portions of this program with the
+  OpenSSL library under certain conditions as described in each
+  individual source file, and distribute linked combinations
+  including the two.
+  You must obey the GNU General Public License in all respects
+  for all of the code used other than OpenSSL.  If you modify
+  file(s) with this exception, you may extend this exception to your
+  version of the file(s), but you are not obligated to do so.  If you
+  do not wish to do so, delete this exception statement from your
+  version.  If you delete this exception statement from all source
+  files in the program, then also delete it here.
 */
 
 #include "eXosip2.h"
@@ -167,6 +180,7 @@ _eXosip_dialog_init_as_uas (eXosip_dialog_t ** _jd, osip_message_t * _invite, os
   jd->d_count = 0;
   jd->d_session_timer_start = 0;
   jd->d_session_timer_length = 0;
+  jd->d_session_timer_use_update = -1;
   jd->d_refresher = -1;         /* 0 -> me / 1 -> remote */
   jd->d_timer = osip_getsystemtime (NULL);
   jd->d_200Ok = NULL;
