@@ -15,6 +15,19 @@
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of portions of this program with the
+  OpenSSL library under certain conditions as described in each
+  individual source file, and distribute linked combinations
+  including the two.
+  You must obey the GNU General Public License in all respects
+  for all of the code used other than OpenSSL.  If you modify
+  file(s) with this exception, you may extend this exception to your
+  version of the file(s), but you are not obligated to do so.  If you
+  do not wish to do so, delete this exception statement from your
+  version.  If you delete this exception statement from all source
+  files in the program, then also delete it here.
 */
 
 #include "eXosip2.h"
@@ -79,8 +92,8 @@ _eXosip_reg_init (struct eXosip_t *excontext, eXosip_reg_t ** jr, const char *fr
     memset (firewall_port, '\0', sizeof (firewall_port));
 
     eXosip_guess_localip (excontext, AF_INET, localip, 128);
-    if (excontext->eXtl != NULL && excontext->eXtl->tl_get_masquerade_contact != NULL) {
-      excontext->eXtl->tl_get_masquerade_contact (excontext, firewall_ip, sizeof (firewall_ip), firewall_port, sizeof (firewall_port));
+    if (excontext->eXtl_transport.tl_get_masquerade_contact != NULL) {
+      excontext->eXtl_transport.tl_get_masquerade_contact (excontext, firewall_ip, sizeof (firewall_ip), firewall_port, sizeof (firewall_port));
     }
 
     osip_MD5Init (&Md5Ctx);
