@@ -15,6 +15,19 @@
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of portions of this program with the
+  OpenSSL library under certain conditions as described in each
+  individual source file, and distribute linked combinations
+  including the two.
+  You must obey the GNU General Public License in all respects
+  for all of the code used other than OpenSSL.  If you modify
+  file(s) with this exception, you may extend this exception to your
+  version of the file(s), but you are not obligated to do so.  If you
+  do not wish to do so, delete this exception statement from your
+  version.  If you delete this exception statement from all source
+  files in the program, then also delete it here.
 */
 
 #include "eXosip2.h"
@@ -29,7 +42,6 @@
 #endif
 #else
 #include <windows.h>
-#include <iphlpapi.h>
 #endif
 
 /* Private functions */
@@ -71,15 +83,15 @@ _eXosip_dialog_add_contact (struct eXosip_t *excontext, osip_message_t * request
   char firewall_port[10];
   int len;
 
-  if (excontext->eXtl == NULL)
+  if (excontext->eXtl_transport.enabled <= 0)
     return OSIP_NO_NETWORK;
   if (request == NULL)
     return OSIP_BADPARAMETER;
 
   firewall_ip[0] = '\0';
   firewall_port[0] = '\0';
-  if (excontext->eXtl->tl_get_masquerade_contact != NULL) {
-    excontext->eXtl->tl_get_masquerade_contact (excontext, firewall_ip, sizeof (firewall_ip), firewall_port, sizeof (firewall_port));
+  if (excontext->eXtl_transport.tl_get_masquerade_contact != NULL) {
+    excontext->eXtl_transport.tl_get_masquerade_contact (excontext, firewall_ip, sizeof (firewall_ip), firewall_port, sizeof (firewall_port));
   }
 
   /* search for topmost Via which indicate the transport protocol */
@@ -133,7 +145,7 @@ _eXosip_dialog_add_contact (struct eXosip_t *excontext, osip_message_t * request
   }
 
   if (locip[0] == '\0') {
-    _eXosip_guess_ip_for_via (excontext, excontext->eXtl->proto_family, locip, 49);
+    _eXosip_guess_ip_for_via (excontext, excontext->eXtl_transport.proto_family, locip, 49);
     if (locip[0] == '\0') {
       OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "eXosip: no default interface defined\n"));
       osip_free (contact);
@@ -141,7 +153,7 @@ _eXosip_dialog_add_contact (struct eXosip_t *excontext, osip_message_t * request
     }
   }
 
-  if (excontext->eXtl->proto_family == AF_INET6) {
+  if (excontext->eXtl_transport.proto_family == AF_INET6) {
     if (a_from->url->username != NULL) {
       char *tmp2 = __osip_uri_escape_userinfo (a_from->url->username);
 
@@ -181,6 +193,9 @@ _eXosip_request_add_via (struct eXosip_t *excontext, osip_message_t * request, c
   char firewall_ip[65];
   char firewall_port[10];
 
+  if (excontext->eXtl_transport.enabled <= 0)
+    return OSIP_NO_NETWORK;
+
   if (request == NULL)
     return OSIP_BADPARAMETER;
 
@@ -197,8 +212,8 @@ _eXosip_request_add_via (struct eXosip_t *excontext, osip_message_t * request, c
 
   firewall_ip[0] = '\0';
   firewall_port[0] = '\0';
-  if (excontext->eXtl != NULL && excontext->eXtl->tl_get_masquerade_contact != NULL) {
-    excontext->eXtl->tl_get_masquerade_contact (excontext, firewall_ip, sizeof (firewall_ip), firewall_port, sizeof (firewall_port));
+  if (excontext->eXtl_transport.tl_get_masquerade_contact != NULL) {
+    excontext->eXtl_transport.tl_get_masquerade_contact (excontext, firewall_ip, sizeof (firewall_ip), firewall_port, sizeof (firewall_port));
   }
 #ifdef MASQUERADE_VIA
   /* this helps to work with a server that don't handle the
@@ -212,7 +227,7 @@ _eXosip_request_add_via (struct eXosip_t *excontext, osip_message_t * request, c
     snprintf (firewall_port, sizeof (firewall_port), "5060");
   }
 
-  if (excontext->eXtl->proto_family == AF_INET6)
+  if (excontext->eXtl_transport.proto_family == AF_INET6)
     snprintf (tmp, 200, "SIP/2.0/%s [%s]:%s;branch=z9hG4bK%u", excontext->transport, ip, firewall_port, osip_build_random_number ());
   else {
     if (excontext->use_rport != 0)
@@ -249,12 +264,12 @@ _eXosip_generating_request_out_of_dialog (struct eXosip_t *excontext, osip_messa
   if (!method || !*method)
     return OSIP_BADPARAMETER;
 
-  if (excontext->eXtl == NULL)
+  if (excontext->eXtl_transport.enabled <= 0)
     return OSIP_NO_NETWORK;
 
   /*guess the local ip since req uri is known */
   memset (locip, '\0', sizeof (locip));
-  _eXosip_guess_ip_for_via (excontext, excontext->eXtl->proto_family, locip, 49);
+  _eXosip_guess_ip_for_via (excontext, excontext->eXtl_transport.proto_family, locip, 49);
   if (locip[0] == '\0') {
     OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "eXosip: no default interface defined\n"));
     return OSIP_NO_NETWORK;
@@ -317,6 +332,24 @@ _eXosip_generating_request_out_of_dialog (struct eXosip_t *excontext, osip_messa
         else if (osip_strcasecmp (u_header->gname, "route") == 0) {
           osip_message_set_route (request, u_header->gvalue);
         }
+        else if (osip_strcasecmp (u_header->gname, "call-info") == 0) {
+          osip_message_set_call_info (request, u_header->gvalue);
+        }
+        else if (osip_strcasecmp (u_header->gname, "accept") == 0) {
+          osip_message_set_accept (request, u_header->gvalue);
+        }
+        else if (osip_strcasecmp (u_header->gname, "accept-encoding") == 0) {
+          osip_message_set_accept_encoding (request, u_header->gvalue);
+        }
+        else if (osip_strcasecmp (u_header->gname, "accept-language") == 0) {
+          osip_message_set_accept_language (request, u_header->gvalue);
+        }
+        else if (osip_strcasecmp (u_header->gname, "alert-info") == 0) {
+          osip_message_set_alert_info (request, u_header->gvalue);
+        }
+        else if (osip_strcasecmp (u_header->gname, "allow") == 0) {
+          osip_message_set_allow (request, u_header->gvalue);
+        }
         else if (osip_strcasecmp (u_header->gname, "content-type") == 0) {
           osip_message_set_content_type (request, u_header->gvalue);
         }
@@ -364,6 +397,24 @@ _eXosip_generating_request_out_of_dialog (struct eXosip_t *excontext, osip_messa
         else if (osip_strcasecmp (u_header->gname, "route") == 0) {
           osip_message_set_route (request, u_header->gvalue);
         }
+        else if (osip_strcasecmp (u_header->gname, "call-info") == 0) {
+          osip_message_set_call_info (request, u_header->gvalue);
+        }
+        else if (osip_strcasecmp (u_header->gname, "accept") == 0) {
+          osip_message_set_accept (request, u_header->gvalue);
+        }
+        else if (osip_strcasecmp (u_header->gname, "accept-encoding") == 0) {
+          osip_message_set_accept_encoding (request, u_header->gvalue);
+        }
+        else if (osip_strcasecmp (u_header->gname, "accept-language") == 0) {
+          osip_message_set_accept_language (request, u_header->gvalue);
+        }
+        else if (osip_strcasecmp (u_header->gname, "alert-info") == 0) {
+          osip_message_set_alert_info (request, u_header->gvalue);
+        }
+        else if (osip_strcasecmp (u_header->gname, "allow") == 0) {
+          osip_message_set_allow (request, u_header->gvalue);
+        }
         else if (osip_strcasecmp (u_header->gname, "content-type") == 0) {
           osip_message_set_content_type (request, u_header->gvalue);
         }
@@ -582,13 +633,13 @@ _eXosip_generating_register (struct eXosip_t *excontext, eXosip_reg_t * jreg, os
   char firewall_ip[65];
   char firewall_port[10];
 
-  if (excontext->eXtl == NULL)
+  if (excontext->eXtl_transport.enabled <= 0)
     return OSIP_NO_NETWORK;
 
   firewall_ip[0] = '\0';
   firewall_port[0] = '\0';
-  if (excontext->eXtl->tl_get_masquerade_contact != NULL) {
-    excontext->eXtl->tl_get_masquerade_contact (excontext, firewall_ip, sizeof (firewall_ip), firewall_port, sizeof (firewall_port));
+  if (excontext->eXtl_transport.tl_get_masquerade_contact != NULL) {
+    excontext->eXtl_transport.tl_get_masquerade_contact (excontext, firewall_ip, sizeof (firewall_ip), firewall_port, sizeof (firewall_port));
   }
 
   i = _eXosip_generating_request_out_of_dialog (excontext, reg, "REGISTER", NULL, transport, from, proxy);
@@ -596,7 +647,7 @@ _eXosip_generating_register (struct eXosip_t *excontext, eXosip_reg_t * jreg, os
     return i;
 
   memset (locip, '\0', sizeof (locip));
-  _eXosip_guess_ip_for_via (excontext, excontext->eXtl->proto_family, locip, 49);
+  _eXosip_guess_ip_for_via (excontext, excontext->eXtl_transport.proto_family, locip, 49);
 
   if (locip[0] == '\0') {
     OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "eXosip: no default interface defined\n"));
@@ -802,13 +853,13 @@ _eXosip_build_request_within_dialog (struct eXosip_t *excontext, osip_message_t
   if (dialog == NULL)
     return OSIP_BADPARAMETER;
 
-  if (excontext->eXtl == NULL)
+  if (excontext->eXtl_transport.enabled <= 0)
     return OSIP_NO_NETWORK;
 
   firewall_ip[0] = '\0';
   firewall_port[0] = '\0';
-  if (excontext->eXtl->tl_get_masquerade_contact != NULL) {
-    excontext->eXtl->tl_get_masquerade_contact (excontext, firewall_ip, sizeof (firewall_ip), firewall_port, sizeof (firewall_port));
+  if (excontext->eXtl_transport.tl_get_masquerade_contact != NULL) {
+    excontext->eXtl_transport.tl_get_masquerade_contact (excontext, firewall_ip, sizeof (firewall_ip), firewall_port, sizeof (firewall_port));
   }
 
   i = osip_message_init (&request);
@@ -825,7 +876,7 @@ _eXosip_build_request_within_dialog (struct eXosip_t *excontext, osip_message_t
 
 
   memset (locip, '\0', sizeof (locip));
-  _eXosip_guess_ip_for_via (excontext, excontext->eXtl->proto_family, locip, 49);
+  _eXosip_guess_ip_for_via (excontext, excontext->eXtl_transport.proto_family, locip, 49);
   if (locip[0] == '\0') {
     OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_ERROR, NULL, "eXosip: no default interface defined\n"));
     osip_message_free (request);
@@ -930,42 +981,7 @@ _eXosip_build_request_within_dialog (struct eXosip_t *excontext, osip_message_t
   }
 
   /* add specific headers for each kind of request... */
-
-#if 0
-  {
-    char contact[200];
-
-    if (firewall_ip[0] != '\0') {
-      char *c_address = request->req_uri->host;
-
-      struct addrinfo *addrinfo;
-      struct __eXosip_sockaddr addr;
-
-      i = _eXosip_get_addrinfo (excontext, &addrinfo, request->req_uri->host, 5060, IPPROTO_UDP);
-      if (i == 0) {
-        memcpy (&addr, addrinfo->ai_addr, addrinfo->ai_addrlen);
-        _eXosip_freeaddrinfo (addrinfo);
-        c_address = inet_ntoa (((struct sockaddr_in *) &addr)->sin_addr);
-        OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO1, NULL, "eXosip: here is the resolved destination host=%s\n", c_address));
-      }
-
-      if (_eXosip_is_public_address (c_address)) {
-        sprintf (contact, "<sip:%s@%s:%s>", dialog->local_uri->url->username, firewall_ip, firewall_port);
-      }
-      else {
-        sprintf (contact, "<sip:%s@%s:%s>", dialog->local_uri->url->username, locip, firewall_port);
-      }
-    }
-    else {
-      sprintf (contact, "<sip:%s@%s:%s>", dialog->local_uri->url->username, locip, firewall_port);
-    }
-    osip_message_set_contact (request, contact);
-    /* Here we'll add the supported header if it's needed! */
-    /* the require header must be added by the upper layer if needed */
-  }
-#else
   _eXosip_dialog_add_contact (excontext, request, NULL);
-#endif
 
   if (0 == strcmp ("NOTIFY", method)) {
   }
